@@ -1,10 +1,8 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 import com.jcraft.jsch.*;
-
-import java.io.*;
 import java.awt.*;
-import java.awt.event.*;
 import javax.swing.*;
+import java.io.*;
 
 public class ScpTo{
   public static void main(String[] arg){
@@ -42,8 +40,6 @@ public class ScpTo{
 
       channel.connect();
 
-      byte[] tmp=new byte[1];
-
       if(checkAck(in)!=0){
 	System.exit(0);
       }
@@ -59,7 +55,6 @@ public class ScpTo{
       }
       command+="\n";
       out.write(command.getBytes()); out.flush();
-
       if(checkAck(in)!=0){
 	System.exit(0);
       }
@@ -70,18 +65,18 @@ public class ScpTo{
       while(true){
         int len=fis.read(buf, 0, buf.length);
 	if(len<=0) break;
-        out.write(buf, 0, len); out.flush();
+        out.write(buf, 0, len); //out.flush();
       }
       fis.close();
       fis=null;
-
       // send '\0'
       buf[0]=0; out.write(buf, 0, 1); out.flush();
-
       if(checkAck(in)!=0){
 	System.exit(0);
       }
+      out.close();
 
+      channel.disconnect();
       session.disconnect();
 
       System.exit(0);
@@ -119,7 +114,7 @@ public class ScpTo{
     return b;
   }
 
-  public static class MyUserInfo implements UserInfo{
+  public static class MyUserInfo implements UserInfo, UIKeyboardInteractive{
     public String getPassword(){ return passwd; }
     public boolean promptYesNo(String str){
       Object[] options={ "yes", "no" };
@@ -151,6 +146,62 @@ public class ScpTo{
     public void showMessage(String message){
       JOptionPane.showMessageDialog(null, message);
     }
-  }
+    final GridBagConstraints gbc = 
+      new GridBagConstraints(0,0,1,1,1,1,
+                             GridBagConstraints.NORTHWEST,
+                             GridBagConstraints.NONE,
+                             new Insets(0,0,0,0),0,0);
+    private Container panel;
+    public String[] promptKeyboardInteractive(String destination,
+                                              String name,
+                                              String instruction,
+                                              String[] prompt,
+                                              boolean[] echo){
+      panel = new JPanel();
+      panel.setLayout(new GridBagLayout());
 
+      gbc.weightx = 1.0;
+      gbc.gridwidth = GridBagConstraints.REMAINDER;
+      gbc.gridx = 0;
+      panel.add(new JLabel(instruction), gbc);
+      gbc.gridy++;
+
+      gbc.gridwidth = GridBagConstraints.RELATIVE;
+
+      JTextField[] texts=new JTextField[prompt.length];
+      for(int i=0; i<prompt.length; i++){
+        gbc.fill = GridBagConstraints.NONE;
+        gbc.gridx = 0;
+        gbc.weightx = 1;
+        panel.add(new JLabel(prompt[i]),gbc);
+
+        gbc.gridx = 1;
+        gbc.fill = GridBagConstraints.HORIZONTAL;
+        gbc.weighty = 1;
+        if(echo[i]){
+          texts[i]=new JTextField(20);
+        }
+        else{
+          texts[i]=new JPasswordField(20);
+        }
+        panel.add(texts[i], gbc);
+        gbc.gridy++;
+      }
+
+      if(JOptionPane.showConfirmDialog(null, panel, 
+                                       destination+": "+name,
+                                       JOptionPane.OK_CANCEL_OPTION,
+                                       JOptionPane.QUESTION_MESSAGE)
+         ==JOptionPane.OK_OPTION){
+        String[] response=new String[prompt.length];
+        for(int i=0; i<prompt.length; i++){
+          response[i]=texts[i].getText();
+        }
+	return response;
+      }
+      else{
+        return null;  // cancel
+      }
+    }
+  }
 }
