@@ -1,7 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 import com.jcraft.jsch.*;
 import java.awt.*;
-import java.awt.event.*;
 import javax.swing.*;
 
 public class Sftp{
@@ -10,13 +9,20 @@ public class Sftp{
     try{
       JSch jsch=new JSch();
 
-      String host=JOptionPane.showInputDialog("Enter username@hostname",
-					      System.getProperty("user.name")+
-					      "@localhost"); 
+      String host=null;
+      if(arg.length>0){
+        host=arg[0];
+      }
+      else{
+        host=JOptionPane.showInputDialog("Enter username@hostname",
+                                         System.getProperty("user.name")+
+                                         "@localhost"); 
+      }
       String user=host.substring(0, host.indexOf('@'));
       host=host.substring(host.indexOf('@')+1);
+      int port=22;
 
-      Session session=jsch.getSession(user, host, 22);
+      Session session=jsch.getSession(user, host, port);
 
       // username and password will be given via UserInfo interface.
       UserInfo ui=new MyUserInfo();
@@ -78,18 +84,17 @@ public class Sftp{
 	  }
 	  try{
 	    level=Integer.parseInt((String)cmds.elementAt(1));
-	    java.util.Hashtable config=new java.util.Hashtable();
 	    if(level==0){
-	      config.put("compression.s2c", "none");
-	      config.put("compression.c2s", "none");
+	      session.setConfig("compression.s2c", "none");
+	      session.setConfig("compression.c2s", "none");
 	    }
 	    else{
-	      config.put("compression.s2c", "zlib,none");
-	      config.put("compression.c2s", "zlib,none");
+              session.setConfig("compression.s2c", "zlib@openssh.com,zlib,none");
+              session.setConfig("compression.c2s", "zlib@openssh.com,zlib,none");
 	    }
-	    session.setConfig(config);
 	  }
 	  catch(Exception e){}
+          session.rekey();
  	  continue;
 	}
 	if(cmd.equals("cd") || cmd.equals("lcd")){
@@ -100,7 +105,7 @@ public class Sftp{
 	    else c.lcd(path);
 	  }
 	  catch(SftpException e){
-	    System.out.println(e.message);
+	    System.out.println(e.toString());
 	  }
 	  continue;
 	}
@@ -113,7 +118,7 @@ public class Sftp{
 	    else c.mkdir(path);
 	  }
 	  catch(SftpException e){
-	    System.out.println(e.message);
+	    System.out.println(e.toString());
 	  }
 	  continue;
 	}
@@ -142,7 +147,7 @@ public class Sftp{
 	    else if(cmd.equals("chmod")){ c.chmod(foo, path); }
 	  }
 	  catch(SftpException e){
-	    System.out.println(e.message);
+	    System.out.println(e.toString());
 	  }
 	  continue;
 	}
@@ -172,7 +177,7 @@ public class Sftp{
 	    }
 	  }
 	  catch(SftpException e){
-	    System.out.println(e.message);
+	    System.out.println(e.toString());
 	  }
 	  continue;
 	}
@@ -225,45 +230,10 @@ public class Sftp{
 	    }
 	  }
 	  catch(SftpException e){
-	    System.out.println(e.message);
+	    System.out.println(e.toString());
 	  }
 	  continue;
 	}
-/*
-	if(cmd.equals("foo")){
-	  try{
-	    java.io.InputStream pis=c.get((String)cmds.elementAt(1));
-	    try{
-	      while(true){
-		int iii=pis.read();
-		if(iii<=0)break;
-		System.out.print(new Character((char)(iii&0xff)));
-	      }
-	    }
-	    catch(Exception ee){
-	      System.out.println("!!!!"+ee);
-	      ee.printStackTrace();
-	    }
-	    pis.close();
-	  }
-	  catch(Exception e){
-	    System.out.println("!!!"+e);
-	  }
-	  continue;
-	}
-	if(cmd.equals("bar")){
-	  try{
-	    java.io.OutputStream pos=c.put((String)cmds.elementAt(1));
-	    pos.write(((String)cmds.elementAt(2)).getBytes());
-	    pos.flush();
-	    pos.close();
-	  }
-	  catch(Exception e){
-	    System.out.println("!!!"+e);
-	  }
-	  continue;
-	}
-*/
 	if(cmd.equals("ln") || cmd.equals("symlink") || cmd.equals("rename")){
           if(cmds.size()!=3) continue;
 	  String p1=(String)cmds.elementAt(1);
@@ -273,7 +243,7 @@ public class Sftp{
 	    else c.symlink(p1, p2);
 	  }
 	  catch(SftpException e){
-	    System.out.println(e.message);
+	    System.out.println(e.toString());
 	  }
 	  continue;
 	}
@@ -286,7 +256,7 @@ public class Sftp{
 	    else attrs=c.lstat(p1);
 	  }
 	  catch(SftpException e){
-	    System.out.println(e.message);
+	    System.out.println(e.toString());
 	  }
 	  if(attrs!=null){
             out.println(attrs);
@@ -295,6 +265,19 @@ public class Sftp{
 	  }
 	  continue;
 	}
+	if(cmd.equals("readlink")){
+          if(cmds.size()!=2) continue;
+	  String p1=(String)cmds.elementAt(1);
+	  String filename=null;
+	  try{
+	    filename=c.readlink(p1);
+            out.println(filename);
+	  }
+	  catch(SftpException e){
+	    System.out.println(e.toString());
+	  }
+	  continue;
+	}
 	if(cmd.equals("version")){
 	  out.println("SFTP protocol version "+c.version());
 	  continue;
@@ -313,7 +296,7 @@ public class Sftp{
     System.exit(0);
   }
 
-  public static class MyUserInfo implements UserInfo{
+  public static class MyUserInfo implements UserInfo, UIKeyboardInteractive{
     public String getPassword(){ return passwd; }
     public boolean promptYesNo(String str){
       Object[] options={ "yes", "no" };
@@ -345,6 +328,63 @@ public class Sftp{
     public void showMessage(String message){
       JOptionPane.showMessageDialog(null, message);
     }
+    final GridBagConstraints gbc = 
+      new GridBagConstraints(0,0,1,1,1,1,
+                             GridBagConstraints.NORTHWEST,
+                             GridBagConstraints.NONE,
+                             new Insets(0,0,0,0),0,0);
+    private Container panel;
+    public String[] promptKeyboardInteractive(String destination,
+                                              String name,
+                                              String instruction,
+                                              String[] prompt,
+                                              boolean[] echo){
+      panel = new JPanel();
+      panel.setLayout(new GridBagLayout());
+
+      gbc.weightx = 1.0;
+      gbc.gridwidth = GridBagConstraints.REMAINDER;
+      gbc.gridx = 0;
+      panel.add(new JLabel(instruction), gbc);
+      gbc.gridy++;
+
+      gbc.gridwidth = GridBagConstraints.RELATIVE;
+
+      JTextField[] texts=new JTextField[prompt.length];
+      for(int i=0; i<prompt.length; i++){
+        gbc.fill = GridBagConstraints.NONE;
+        gbc.gridx = 0;
+        gbc.weightx = 1;
+        panel.add(new JLabel(prompt[i]),gbc);
+
+        gbc.gridx = 1;
+        gbc.fill = GridBagConstraints.HORIZONTAL;
+        gbc.weighty = 1;
+        if(echo[i]){
+          texts[i]=new JTextField(20);
+        }
+        else{
+          texts[i]=new JPasswordField(20);
+        }
+        panel.add(texts[i], gbc);
+        gbc.gridy++;
+      }
+
+      if(JOptionPane.showConfirmDialog(null, panel, 
+                                       destination+": "+name,
+                                       JOptionPane.OK_CANCEL_OPTION,
+                                       JOptionPane.QUESTION_MESSAGE)
+         ==JOptionPane.OK_OPTION){
+        String[] response=new String[prompt.length];
+        for(int i=0; i<prompt.length; i++){
+          response[i]=texts[i].getText();
+        }
+	return response;
+      }
+      else{
+        return null;  // cancel
+      }
+    }
   }
 
 /*
@@ -450,6 +490,7 @@ public class Sftp{
 "rmdir path                    Remove remote directory\n"+
 "rm path                       Delete remote file\n"+
 "symlink oldpath newpath       Symlink remote file\n"+
+"readlink path                 Check the target of a symbolic link\n"+
 "rekey                         Key re-exchanging\n"+
 "compression level             Packet compression will be enabled\n"+
 "version                       Show SFTP version\n"+
