@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -29,60 +29,47 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 package com.jcraft.jsch;
 
-import java.net.*;
 import java.util.*;
 
 public class ChannelExec extends ChannelSession{
-  boolean xforwading=false;
-  boolean pty=false;
-  Hashtable env=null;
+
   String command="";
-  public void setXForwarding(boolean foo){ xforwading=foo; }
-  public void setPty(boolean foo){ pty=foo; }
-  public void setEnv(Hashtable foo){ env=foo; }
+
   public void start() throws JSchException{
     try{
-      Request request;
-
-      if(xforwading){
-        request=new RequestX11();
-        request.request(session, this);
-      }
-
-      if(pty){
-	request=new RequestPtyReq();
-	request.request(session, this);
-      }
-
-      if(env!=null){
-        for(Enumeration _env=env.keys() ; _env.hasMoreElements() ;) {
-          String name=(String)(_env.nextElement());
-          String value=(String)(env.get(name));
-          request=new RequestEnv();
-          ((RequestEnv)request).setEnv(name, value);
-          request.request(session, this);
-        }
-      }
-
-      request=new RequestExec(command);
+      sendRequests();
+      Request request=new RequestExec(command);
       request.request(session, this);
     }
     catch(Exception e){
+      if(e instanceof JSchException) throw (JSchException)e;
+      if(e instanceof Throwable)
+        throw new JSchException("ChannelExec", (Throwable)e);
       throw new JSchException("ChannelExec");
     }
-    thread=new Thread(this);
-    thread.setName("Exec thread "+session.getHost());
-    thread.start();
+
+    if(io.in!=null){
+      thread=new Thread(this);
+      thread.setName("Exec thread "+session.getHost());
+      if(session.daemon_thread){
+        thread.setDaemon(session.daemon_thread);
+      }
+      thread.start();
+    }
   }
+
   public void setCommand(String foo){ command=foo;}
   public void init(){
     io.setInputStream(session.in);
     io.setOutputStream(session.out);
   }
-  //public void finalize() throws java.lang.Throwable{ super.finalize(); }
+
   public void setErrStream(java.io.OutputStream out){
     setExtOutputStream(out);
   }
+  public void setErrStream(java.io.OutputStream out, boolean dontclose){
+    setExtOutputStream(out, dontclose);
+  }
   public java.io.InputStream getErrStream() throws java.io.IOException {
     return getExtInputStream();
   }
