@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -30,24 +30,30 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 package com.jcraft.jsch;
 
 import java.net.*;
+
 class ChannelX11 extends Channel{
 
   static private final int LOCAL_WINDOW_SIZE_MAX=0x20000;
   static private final int LOCAL_MAXIMUM_PACKET_SIZE=0x4000;
 
-  static String host="127.0.0.1";
-  static int port=6000;
+  static private final int TIMEOUT=10*1000;
 
-  boolean init=true;
+  private static String host="127.0.0.1";
+  private static int port=6000;
+
+  private boolean init=true;
 
   static byte[] cookie=null;
-  static byte[] cookie_hex=null;
+  private static byte[] cookie_hex=null;
 
   private static java.util.Hashtable faked_cookie_pool=new java.util.Hashtable();
   private static java.util.Hashtable faked_cookie_hex_pool=new java.util.Hashtable();
 
-  static byte[] table={0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,
-                        0x61,0x62,0x63,0x64,0x65,0x66};
+  private static byte[] table={0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,
+                               0x61,0x62,0x63,0x64,0x65,0x66};
+
+  private Socket socket = null;
+
   static int revtable(byte foo){
     for(int i=0; i<table.length; i++){
       if(table[i]==foo)return i;
@@ -68,17 +74,17 @@ class ChannelX11 extends Channel{
     synchronized(faked_cookie_hex_pool){
       byte[] foo=(byte[])faked_cookie_hex_pool.get(session);
       if(foo==null){
-	Random random=session.random;
+	Random random=Session.random;
 	foo=new byte[16];
 	synchronized(random){
 	  random.fill(foo, 0, 16);
 	}
 /*
-System.out.print("faked_cookie: ");
+System.err.print("faked_cookie: ");
 for(int i=0; i<foo.length; i++){
-    System.out.print(Integer.toHexString(foo[i]&0xff)+":");
+    System.err.print(Integer.toHexString(foo[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
 	faked_cookie_pool.put(session, foo);
 	byte[] bar=new byte[32];
@@ -93,7 +99,6 @@ System.out.println("");
     }
   }
 
-  Socket socket = null;
   ChannelX11(){
     super();
 
@@ -102,25 +107,47 @@ System.out.println("");
     setLocalPacketSize(LOCAL_MAXIMUM_PACKET_SIZE);
 
     type="x11".getBytes();
+
+    connected=true;
+    /*
     try{ 
-      socket=new Socket(host, port);
+      socket=Util.createSocket(host, port, TIMEOUT);
       socket.setTcpNoDelay(true);
       io=new IO();
       io.setInputStream(socket.getInputStream());
       io.setOutputStream(socket.getOutputStream());
     }
     catch(Exception e){
-      System.out.println(e);
+      //System.err.println(e);
     }
+    */
   }
 
   public void run(){
+
+    try{ 
+      socket=Util.createSocket(host, port, TIMEOUT);
+      socket.setTcpNoDelay(true);
+      io=new IO();
+      io.setInputStream(socket.getInputStream());
+      io.setOutputStream(socket.getOutputStream());
+      sendOpenConfirmation();
+    }
+    catch(Exception e){
+      sendOpenFailure(SSH_OPEN_ADMINISTRATIVELY_PROHIBITED);
+      close=true;
+      disconnect();
+      return;
+    }
+
     thread=Thread.currentThread();
     Buffer buf=new Buffer(rmpsize);
     Packet packet=new Packet(buf);
     int i=0;
     try{
-      while(thread!=null){
+      while(thread!=null &&
+            io!=null &&
+            io.in!=null){
         i=io.in.read(buf.buffer, 
 		     14, 
 		     buf.buffer.length-14
@@ -140,18 +167,36 @@ System.out.println("");
       }
     }
     catch(Exception e){
-      //System.out.println(e);
+      //System.err.println(e);
     }
-//    thread.notifyAll();
-//    thread=null;
+    disconnect();
+  }
+
+  private byte[] cache=new byte[0];
+  private byte[] addCache(byte[] foo, int s, int l){
+    byte[] bar=new byte[cache.length+l];
+    System.arraycopy(foo, s, bar, cache.length, l);
+    if(cache.length>0)
+      System.arraycopy(cache, 0, bar, 0, cache.length);
+    cache=bar;
+    return cache;
   }
 
   void write(byte[] foo, int s, int l) throws java.io.IOException {
     //if(eof_local)return;
 
     if(init){
+
+      foo=addCache(foo, s, l);
+      s=0; 
+      l=foo.length;
+
+      if(l<9)
+        return;
+
       int plen=(foo[s+6]&0xff)*256+(foo[s+7]&0xff);
       int dlen=(foo[s+8]&0xff)*256+(foo[s+9]&0xff);
+
       if((foo[s]&0xff)==0x42){
       }
       else if((foo[s]&0xff)==0x6c){
@@ -161,6 +206,10 @@ System.out.println("");
       else{
 	  // ??
       }
+
+      if(l<12+plen+((-plen)&3)+dlen)
+        return;
+
       byte[] bar=new byte[dlen];
       System.arraycopy(foo, s+12+plen+((-plen)&3), bar, 0, dlen);
       byte[] faked_cookie=null;
@@ -170,29 +219,33 @@ System.out.println("");
       }
 
       /*
-System.out.print("faked_cookie: ");
+System.err.print("faked_cookie: ");
 for(int i=0; i<faked_cookie.length; i++){
-    System.out.print(Integer.toHexString(faked_cookie[i]&0xff)+":");
+    System.err.print(Integer.toHexString(faked_cookie[i]&0xff)+":");
 }
-System.out.println("");
-System.out.print("bar: ");
+System.err.println("");
+System.err.print("bar: ");
 for(int i=0; i<bar.length; i++){
-    System.out.print(Integer.toHexString(bar[i]&0xff)+":");
+    System.err.print(Integer.toHexString(bar[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
       */
+
       if(equals(bar, faked_cookie)){
         if(cookie!=null)
           System.arraycopy(cookie, 0, foo, s+12+plen+((-plen)&3), dlen);
       }
       else{
-	  //System.out.println("wrong cookie");
+	  //System.err.println("wrong cookie");
           thread=null;
           eof();
           io.close();
           disconnect();
       }
       init=false;
+      io.put(foo, s, l);
+      cache=null;
+      return;
     }
     io.put(foo, s, l);
   }
