@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -33,22 +33,32 @@ public class HostKey{
   private static final byte[] sshdss="ssh-dss".getBytes();
   private static final byte[] sshrsa="ssh-rsa".getBytes();
 
-  public static final int SSHDSS=0;
-  public static final int SSHRSA=1;
-  static final int UNKNOWN=2;
+  protected static final int GUESS=0;
+  public static final int SSHDSS=1;
+  public static final int SSHRSA=2;
+  static final int UNKNOWN=3;
+
+  protected String host;
+  protected int type;
+  protected byte[] key;
 
-  String host;
-  int type;
-  byte[] key;
   public HostKey(String host, byte[] key) throws JSchException {
-    this.host=host; this.key=key;
-    if(key[8]=='d'){ this.type=SSHDSS; }
-    else if(key[8]=='r'){ this.type=SSHRSA; }
-    else { throw new JSchException("invalid key type");}
+    this(host, GUESS, key);
   }
-  HostKey(String host, int type, byte[] key){
-    this.host=host; this.type=type; this.key=key;
+
+  public HostKey(String host, int type, byte[] key) throws JSchException {
+    this.host=host; 
+    if(type==GUESS){
+      if(key[8]=='d'){ this.type=SSHDSS; }
+      else if(key[8]=='r'){ this.type=SSHRSA; }
+      else { throw new JSchException("invalid key type");}
+    }
+    else{
+      this.type=type; 
+    }
+    this.key=key;
   }
+
   public String getHost(){ return host; }
   public String getType(){
     if(type==SSHDSS){ return new String(sshdss); }
@@ -67,4 +77,28 @@ public class HostKey{
     catch(Exception e){ System.err.println("getFingerPrint: "+e); }
     return Util.getFingerPrint(hash, key);
   }
+
+  boolean isMatched(String _host){
+    return isIncluded(_host);
+  }
+
+  private boolean isIncluded(String _host){
+    int i=0;
+    String hosts=this.host; 
+    int hostslen=hosts.length();
+    int hostlen=_host.length();
+    int j;
+    while(i<hostslen){
+      j=hosts.indexOf(',', i);
+      if(j==-1){
+       if(hostlen!=hostslen-i) return false;
+       return hosts.regionMatches(true, i, _host, 0, hostlen);
+      }
+      if(hostlen==(j-i)){
+	if(hosts.regionMatches(true, i, _host, 0, hostlen)) return true;
+      }
+      i=j+1;
+    }
+    return false;
+  }
 }
