@@ -1,6 +1,6 @@
-/* -*-mode:java; c-basic-offset:2; -*- */
+/* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -72,23 +72,88 @@ class IdentityFile implements Identity{
 
   private boolean encrypted=true;
 
-  IdentityFile(String identity, JSch jsch) throws JSchException{
-    this.identity=identity;
+  static IdentityFile newInstance(String prvfile, String pubfile, JSch jsch) throws JSchException{
+    byte[] prvkey=null;
+    byte[] pubkey=null;
+
+    File file=null;
+    FileInputStream fis=null;
+    try{
+      file=new File(prvfile);
+      fis=new FileInputStream(prvfile);
+      prvkey=new byte[(int)(file.length())];
+      int len=0;
+      while(true){
+        int i=fis.read(prvkey, len, prvkey.length-len);
+        if(i<=0)
+          break;
+        len+=i;
+      }
+      fis.close();
+    }
+    catch(Exception e){
+      try{ if(fis!=null) fis.close();}
+      catch(Exception ee){}
+      if(e instanceof Throwable)
+        throw new JSchException(e.toString(), (Throwable)e);
+      throw new JSchException(e.toString());
+    }
+
+    String _pubfile=pubfile;
+    if(pubfile==null){
+      _pubfile=prvfile+".pub";
+    }
+
+    try{
+      file=new File(_pubfile);
+      fis = new FileInputStream(_pubfile);
+      pubkey=new byte[(int)(file.length())];
+      int len=0;
+      while(true){
+        int i=fis.read(pubkey, len, pubkey.length-len);
+        if(i<=0)
+          break;
+        len+=i;
+      }
+      fis.close();
+    }
+    catch(Exception e){
+      try{ if(fis!=null) fis.close();}
+      catch(Exception ee){}
+      if(pubfile!=null){  
+        // The pubfile is explicitry given, but not accessible.
+        if(e instanceof Throwable)
+          throw new JSchException(e.toString(), (Throwable)e);
+        throw new JSchException(e.toString());
+      }
+    }
+    return newInstance(prvfile, prvkey, pubkey, jsch);
+  }
+
+  static IdentityFile newInstance(String name, byte[] prvkey, byte[] pubkey, JSch jsch) throws JSchException{
+    try{
+      return new IdentityFile(name, prvkey, pubkey, jsch);
+    }
+    finally{
+      Util.bzero(prvkey);
+    }
+  }
+
+  private IdentityFile(String name, byte[] prvkey, byte[] pubkey, JSch jsch) throws JSchException{
+    this.identity=name;
     this.jsch=jsch;
     try{
       Class c;
-      c=Class.forName(jsch.getConfig("3des-cbc"));
+      c=Class.forName((String)jsch.getConfig("3des-cbc"));
       cipher=(Cipher)(c.newInstance());
       key=new byte[cipher.getBlockSize()];   // 24
       iv=new byte[cipher.getIVSize()];       // 8
-      c=Class.forName(jsch.getConfig("md5"));
+      c=Class.forName((String)jsch.getConfig("md5"));
       hash=(HASH)(c.newInstance());
       hash.init();
-      File file=new File(identity);
-      FileInputStream fis = new FileInputStream(identity);
-      byte[] buf=new byte[(int)(file.length())];
-      int len=fis.read(buf, 0, buf.length);
-      fis.close();
+
+      byte[] buf=prvkey;
+      int len=buf.length;
 
       int i=0;
       while(i<len){
@@ -101,8 +166,8 @@ class IdentityFile implements Identity{
 	    keytype=FSECURE;
 	  }
 	  else{
-            //System.out.println("invalid format: "+identity);
-	    throw new JSchException("invaid privatekey: "+identity);
+            //System.err.println("invalid format: "+identity);
+	    throw new JSchException("invalid privatekey: "+identity);
 	  }
           i+=3;
 	  continue;
@@ -142,7 +207,7 @@ class IdentityFile implements Identity{
       }
 
       if(type==ERROR){
-	throw new JSchException("invaid privatekey: "+identity);
+	throw new JSchException("invalid privatekey: "+identity);
       }
 
       int start=i;
@@ -173,10 +238,10 @@ class IdentityFile implements Identity{
 	_buf.getInt();  // 0x3f6ff9be
 	_buf.getInt();
 	byte[]_type=_buf.getString();
-	//System.out.println("type: "+new String(_type)); 
+	//System.err.println("type: "+new String(_type)); 
 	byte[] _cipher=_buf.getString();
 	String cipher=new String(_cipher);
-	//System.out.println("cipher: "+cipher); 
+	//System.err.println("cipher: "+cipher); 
 	if(cipher.equals("3des-cbc")){
   	   _buf.getInt();
 	   byte[] foo=new byte[encoded_data.length-_buf.getOffSet()];
@@ -198,28 +263,22 @@ class IdentityFile implements Identity{
 
       }
 
-      try{
-        file=new File(identity+".pub");
-        fis=new FileInputStream(identity+".pub");
-        buf=new byte[(int)(file.length())];
-        len=fis.read(buf, 0, buf.length);
-        fis.close();
-      }
-      catch(Exception ee){
-	return;
+      if(pubkey==null){
+        return;
       }
+      
+      buf=pubkey;
+      len=buf.length;
 
       if(buf.length>4 &&             // FSecure's public key
 	 buf[0]=='-' && buf[1]=='-' && buf[2]=='-' && buf[3]=='-'){
-
 	i=0;
-	do{i++;}while(buf.length>i && buf[i]!=0x0a);
-	if(buf.length<=i) return;
-
-	while(true){
+	do{i++;}while(len>i && buf[i]!=0x0a);
+	if(len<=i) return;
+	while(i<len){
 	  if(buf[i]==0x0a){
 	    boolean inheader=false;
-	    for(int j=i+1; j<buf.length; j++){
+	    for(int j=i+1; j<len; j++){
 	      if(buf[j]==0x0a) break;
 	      if(buf[j]==':'){inheader=true; break;}
 	    }
@@ -230,7 +289,7 @@ class IdentityFile implements Identity{
 	  }
 	  i++;
 	}
-	if(buf.length<=i) return;
+	if(len<=i) return;
 
 	start=i;
 	while(i<len){
@@ -244,7 +303,7 @@ class IdentityFile implements Identity{
 	}
 	publickeyblob=Util.fromBase64(buf, start, i-start);
 
-	if(type==UNKNOWN){
+	if(type==UNKNOWN && publickeyblob.length>8){
 	  if(publickeyblob[8]=='d'){
 	    type=DSS;
 	  }
@@ -259,17 +318,24 @@ class IdentityFile implements Identity{
 	while(i<len){ if(buf[i]==' ')break; i++;} i++;
 	if(i>=len) return;
 	start=i;
-	while(i<len){ if(buf[i]==' ')break; i++;}
+	while(i<len){ if(buf[i]==' ' || buf[i]=='\n')break; i++;}
 	publickeyblob=Util.fromBase64(buf, start, i-start);
+        if(publickeyblob.length<4+7){  // It must start with "ssh-XXX".
+          if(JSch.getLogger().isEnabled(Logger.WARN)){
+            JSch.getLogger().log(Logger.WARN, 
+                                 "failed to parse the public key");
+          }
+          publickeyblob=null;
+        }
       }
-
     }
     catch(Exception e){
-      System.out.println("Identity: "+e);
+      //System.err.println("IdentityFile: "+e);
       if(e instanceof JSchException) throw (JSchException)e;
+      if(e instanceof Throwable)
+        throw new JSchException(e.toString(), (Throwable)e);
       throw new JSchException(e.toString());
     }
-
   }
 
   public String getAlgName(){
@@ -277,7 +343,7 @@ class IdentityFile implements Identity{
     return "ssh-dss"; 
   }
 
-  public boolean setPassphrase(String _passphrase) throws JSchException{
+  public boolean setPassphrase(byte[] _passphrase) throws JSchException{
     /*
       hash is MD5
       h(0) <- hash(passphrase, iv);
@@ -287,7 +353,7 @@ class IdentityFile implements Identity{
     try{
       if(encrypted){
 	if(_passphrase==null) return false;
-	byte[] passphrase=_passphrase.getBytes();
+	byte[] passphrase=_passphrase;
 	int hsize=hash.getBlockSize();
 	byte[] hn=new byte[key.length/hsize*hsize+
 			   (key.length%hsize==0?0:hsize)];
@@ -313,6 +379,7 @@ class IdentityFile implements Identity{
 	  }
 	  System.arraycopy(hn, 0, key, 0, key.length); 
 	}
+        Util.bzero(passphrase);
       }
       if(decrypt()){
 	encrypted=false;
@@ -323,6 +390,8 @@ class IdentityFile implements Identity{
     }
     catch(Exception e){
       if(e instanceof JSchException) throw (JSchException)e;
+      if(e instanceof Throwable)
+        throw new JSchException(e.toString(), (Throwable)e);
       throw new JSchException(e.toString());
     }
   }
@@ -359,28 +428,19 @@ class IdentityFile implements Identity{
     return buf.buffer;
   }
 
-  public byte[] getSignature(Session session, byte[] data){
-    if(type==RSA) return getSignature_rsa(session, data);
-    return getSignature_dss(session, data);
+  public byte[] getSignature(byte[] data){
+    if(type==RSA) return getSignature_rsa(data);
+    return getSignature_dss(data);
   }
 
-  byte[] getSignature_rsa(Session session, byte[] data){
+  byte[] getSignature_rsa(byte[] data){
     try{      
-      Class c=Class.forName(jsch.getConfig("signature.rsa"));
+      Class c=Class.forName((String)jsch.getConfig("signature.rsa"));
       SignatureRSA rsa=(SignatureRSA)(c.newInstance());
 
       rsa.init();
       rsa.setPrvKey(d_array, n_array);
 
-      /*
-      byte[] goo=new byte[4];
-      goo[0]=(byte)(session.getSessionId().length>>>24);
-      goo[1]=(byte)(session.getSessionId().length>>>16);
-      goo[2]=(byte)(session.getSessionId().length>>>8);
-      goo[3]=(byte)(session.getSessionId().length);
-      rsa.update(goo);
-      rsa.update(session.getSessionId());
-      */
       rsa.update(data);
       byte[] sig = rsa.sign();
       Buffer buf=new Buffer("ssh-rsa".length()+4+
@@ -394,45 +454,36 @@ class IdentityFile implements Identity{
     return null;
   }
 
-  byte[] getSignature_dss(Session session, byte[] data){
+  byte[] getSignature_dss(byte[] data){
 /*
     byte[] foo;
     int i;
-    System.out.print("P ");
+    System.err.print("P ");
     foo=P_array;
     for(i=0;  i<foo.length; i++){
-      System.out.print(Integer.toHexString(foo[i]&0xff)+":");
+      System.err.print(Integer.toHexString(foo[i]&0xff)+":");
     }
-    System.out.println("");
-    System.out.print("Q ");
+    System.err.println("");
+    System.err.print("Q ");
     foo=Q_array;
     for(i=0;  i<foo.length; i++){
-      System.out.print(Integer.toHexString(foo[i]&0xff)+":");
+      System.err.print(Integer.toHexString(foo[i]&0xff)+":");
     }
-    System.out.println("");
-    System.out.print("G ");
+    System.err.println("");
+    System.err.print("G ");
     foo=G_array;
     for(i=0;  i<foo.length; i++){
-      System.out.print(Integer.toHexString(foo[i]&0xff)+":");
+      System.err.print(Integer.toHexString(foo[i]&0xff)+":");
     }
-    System.out.println("");
+    System.err.println("");
 */
 
     try{      
-      Class c=Class.forName(jsch.getConfig("signature.dss"));
+      Class c=Class.forName((String)jsch.getConfig("signature.dss"));
       SignatureDSA dsa=(SignatureDSA)(c.newInstance());
       dsa.init();
       dsa.setPrvKey(prv_array, P_array, Q_array, G_array);
 
-      /*
-      byte[] goo=new byte[4];
-      goo[0]=(byte)(session.getSessionId().length>>>24);
-      goo[1]=(byte)(session.getSessionId().length>>>16);
-      goo[2]=(byte)(session.getSessionId().length>>>8);
-      goo[3]=(byte)(session.getSessionId().length);
-      dsa.update(goo);
-      dsa.update(session.getSessionId());
-      */
       dsa.update(data);
       byte[] sig = dsa.sign();
       Buffer buf=new Buffer("ssh-dss".length()+4+
@@ -442,7 +493,7 @@ class IdentityFile implements Identity{
       return buf.buffer;
     }
     catch(Exception e){
-      System.out.println("e "+e);
+      //System.err.println("e "+e);
     }
     return null;
   }
@@ -517,9 +568,9 @@ class IdentityFile implements Identity{
       }
       index+=length;
 
-//System.out.println("int: len="+length);
-//System.out.print(Integer.toHexString(plain[index-1]&0xff)+":");
-//System.out.println("");
+//System.err.println("int: len="+length);
+//System.err.print(Integer.toHexString(plain[index-1]&0xff)+":");
+//System.err.println("");
 
       index++;
       length=plain[index++]&0xff;
@@ -531,11 +582,11 @@ class IdentityFile implements Identity{
       System.arraycopy(plain, index, n_array, 0, length);
       index+=length;
 /*
-System.out.println("int: N len="+length);
+System.err.println("int: N len="+length);
 for(int i=0; i<n_array.length; i++){
-System.out.print(Integer.toHexString(n_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(n_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
       index++;
       length=plain[index++]&0xff;
@@ -547,11 +598,11 @@ System.out.println("");
       System.arraycopy(plain, index, e_array, 0, length);
       index+=length;
 /*
-System.out.println("int: E len="+length);
+System.err.println("int: E len="+length);
 for(int i=0; i<e_array.length; i++){
-System.out.print(Integer.toHexString(e_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(e_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
       index++;
       length=plain[index++]&0xff;
@@ -563,11 +614,11 @@ System.out.println("");
       System.arraycopy(plain, index, d_array, 0, length);
       index+=length;
 /*
-System.out.println("int: D len="+length);
+System.err.println("int: D len="+length);
 for(int i=0; i<d_array.length; i++){
-System.out.print(Integer.toHexString(d_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(d_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
 
       index++;
@@ -580,11 +631,11 @@ System.out.println("");
       System.arraycopy(plain, index, p_array, 0, length);
       index+=length;
 /*
-System.out.println("int: P len="+length);
+System.err.println("int: P len="+length);
 for(int i=0; i<p_array.length; i++){
-System.out.print(Integer.toHexString(p_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(p_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
       index++;
       length=plain[index++]&0xff;
@@ -596,11 +647,11 @@ System.out.println("");
       System.arraycopy(plain, index, q_array, 0, length);
       index+=length;
 /*
-System.out.println("int: q len="+length);
+System.err.println("int: q len="+length);
 for(int i=0; i<q_array.length; i++){
-System.out.print(Integer.toHexString(q_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(q_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
       index++;
       length=plain[index++]&0xff;
@@ -612,11 +663,11 @@ System.out.println("");
       System.arraycopy(plain, index, dmp1_array, 0, length);
       index+=length;
 /*
-System.out.println("int: dmp1 len="+length);
+System.err.println("int: dmp1 len="+length);
 for(int i=0; i<dmp1_array.length; i++){
-System.out.print(Integer.toHexString(dmp1_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(dmp1_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
       index++;
       length=plain[index++]&0xff;
@@ -628,11 +679,11 @@ System.out.println("");
       System.arraycopy(plain, index, dmq1_array, 0, length);
       index+=length;
 /*
-System.out.println("int: dmq1 len="+length);
+System.err.println("int: dmq1 len="+length);
 for(int i=0; i<dmq1_array.length; i++){
-System.out.print(Integer.toHexString(dmq1_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(dmq1_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
       index++;
       length=plain[index++]&0xff;
@@ -644,15 +695,15 @@ System.out.println("");
       System.arraycopy(plain, index, iqmp_array, 0, length);
       index+=length;
 /*
-System.out.println("int: iqmp len="+length);
+System.err.println("int: iqmp len="+length);
 for(int i=0; i<iqmp_array.length; i++){
-System.out.print(Integer.toHexString(iqmp_array[i]&0xff)+":");
+System.err.print(Integer.toHexString(iqmp_array[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
     }
     catch(Exception e){
-      //System.out.println(e);
+      //System.err.println(e);
       return false;
     }
     return true;
@@ -668,9 +719,9 @@ System.out.println("");
 	  cipher.update(encoded_data, 0, encoded_data.length, plain, 0);
 /*
 for(int i=0; i<plain.length; i++){
-System.out.print(Integer.toHexString(plain[i]&0xff)+":");
+System.err.print(Integer.toHexString(plain[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
 	}
 	else if(keytype==FSECURE){
@@ -704,7 +755,6 @@ System.out.println("");
 
       int index=0;
       int length=0;
-
       if(plain[index]!=0x30)return false;
       index++; // SEQUENCE
       length=plain[index++]&0xff;
@@ -712,7 +762,6 @@ System.out.println("");
         int foo=length&0x7f; length=0;
         while(foo-->0){ length=(length<<8)+(plain[index++]&0xff); }
       }
-
       if(plain[index]!=0x02)return false;
       index++; // INTEGER
       length=plain[index++]&0xff;
@@ -773,7 +822,7 @@ System.out.println("");
       index+=length;
     }
     catch(Exception e){
-      //System.out.println(e);
+      //System.err.println(e);
       //e.printStackTrace();
       return false;
     }
@@ -783,45 +832,9 @@ System.out.println("");
   public boolean isEncrypted(){
     return encrypted;
   }
-  public String getName(){return identity;}
 
-  private int writeSEQUENCE(byte[] buf, int index, int len){
-    buf[index++]=0x30;
-    index=writeLength(buf, index, len);
-    return index;
-  }
-  private int writeINTEGER(byte[] buf, int index, byte[] data){
-    buf[index++]=0x02;
-    index=writeLength(buf, index, data.length);
-    System.arraycopy(data, 0, buf, index, data.length);
-    index+=data.length;
-    return index;
-  }
-
-  private int countLength(int len){
-    int i=1;
-    if(len<=0x7f) return i;
-    while(len>0){
-      len>>>=8;
-      i++;
-    }
-    return i;
-  }
-
-  private int writeLength(byte[] data, int index, int len){
-    int i=countLength(len)-1;
-    if(i==0){
-      data[index++]=(byte)len;
-      return index;
-    }
-    data[index++]=(byte)(0x80|i);
-    int j=index+i;
-    while(i>0){
-      data[index+i-1]=(byte)(len&0xff);
-      len>>>=8;
-      i--;
-    }
-    return j;
+  public String getName(){
+    return identity;
   }
 
   private byte a2b(byte c){
@@ -829,8 +842,22 @@ System.out.println("");
     if('a'<=c&&c<='z') return (byte)(c-'a'+10);
     return (byte)(c-'A'+10);
   }
-  private byte b2a(byte c){
-    if(0<=c&&c<=9) return (byte)(c+'0');
-    return (byte)(c-10+'A');
+
+  public boolean equals(Object o){
+    if(!(o instanceof IdentityFile)) return super.equals(o);
+    IdentityFile foo=(IdentityFile)o;
+    return getName().equals(foo.getName());
+  }
+
+  public void clear(){
+    Util.bzero(encoded_data);
+    Util.bzero(prv_array);
+    Util.bzero(d_array);
+    Util.bzero(key);
+    Util.bzero(iv);
+  }
+
+  public void finalize (){
+    clear();
   }
 }
