@@ -1,6 +1,6 @@
 /* -*-mode:java; c-basic-offset:2; indent-tabs-mode:nil -*- */
 /*
-Copyright (c) 2002,2003,2004,2005,2006 ymnk, JCraft,Inc. All rights reserved.
+Copyright (c) 2002-2008 ymnk, JCraft,Inc. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
@@ -35,7 +35,7 @@ public class Packet{
   static void setRandom(Random foo){ random=foo;}
 
   Buffer buffer;
-  byte[] tmp=new byte[4]; 
+  byte[] ba4=new byte[4]; 
   public Packet(Buffer buffer){
     this.buffer=buffer;
   }
@@ -49,11 +49,11 @@ public class Packet{
       pad+=bsize;
     }
     len=len+pad-4;
-    tmp[0]=(byte)(len>>>24);
-    tmp[1]=(byte)(len>>>16);
-    tmp[2]=(byte)(len>>>8);
-    tmp[3]=(byte)(len);
-    System.arraycopy(tmp, 0, buffer.buffer, 0, 4);
+    ba4[0]=(byte)(len>>>24);
+    ba4[1]=(byte)(len>>>16);
+    ba4[2]=(byte)(len>>>8);
+    ba4[3]=(byte)(len);
+    System.arraycopy(ba4, 0, buffer.buffer, 0, 4);
     buffer.buffer[4]=(byte)pad;
     synchronized(random){
       random.fill(buffer.buffer, buffer.index, pad);
@@ -62,9 +62,9 @@ public class Packet{
     //buffer.putPad(pad);
 /*
 for(int i=0; i<buffer.index; i++){
-System.out.print(Integer.toHexString(buffer.buffer[i]&0xff)+":");
+System.err.print(Integer.toHexString(buffer.buffer[i]&0xff)+":");
 }
-System.out.println("");
+System.err.println("");
 */
   }
 
@@ -108,4 +108,7 @@ System.out.println("");
     buffer.putInt(len);
     buffer.index=len+5+9;
   }
+  Buffer getBuffer(){
+    return buffer;
+  }
 }
